/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.tools.channelclient.jmxclient;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.InstanceNotFoundException;
import javax.management.ListenerNotFoundException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.NotificationFilterSupport;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.security.auth.Subject;

import org.ow2.petals.jmx.api.IPetalsAdminService;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientDoesNotExistException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientServiceErrorException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.PerformActionErrorException;

/**
 * Client API of the JMX Sample Client service.
 * 
 * @author mcarpentier - EBM WebSourcing
 * 
 */
public class ChannelClientServiceClient extends ChannelClientAbstractServiceClient implements
        IPetalsAdminService {

    public final static String SAMPLECLIENT_JMX_NAME = "ChannelClient";

    public final static String SEND_SYNC = "sendSync";

    public final static String SEND_ASYNC = "sendASync";

    /**
     * Creates a new instance of {@link ChannelClientServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     */
    public ChannelClientServiceClient(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws ChannelClientDoesNotExistException, ChannelClientServiceErrorException {

        super(petalsDomain, beanServerConnection);
        try {
            Set<ObjectName> objNames;

            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, SAMPLECLIENT_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new ChannelClientDoesNotExistException("Can't find Sample Client Service");
            }
        } catch (MalformedObjectNameException e) {
            throw new ChannelClientServiceErrorException(e);
        } catch (IOException e) {
            throw new ChannelClientServiceErrorException(e);
        }
    }

    /**
     * Send a message to PEtALS asynchronously. You need a listener to get the
     * response back
     * 
     * @param callBackListener
     *            a listener to get the response back
     * @param interfaceName
     * @param serviceName
     * @param operationName
     * @param endpointName
     * @param mep
     * @param content
     * @param attachmentFiles
     * @param properties
     * @param subject
     * @throws PerformActionErrorException
     * @throws ChannelClientDoesNotExistException
     */
    @SuppressWarnings("unchecked")
    public String sendASync(NotificationListener callBackListener, final String interfaceName,
            final String serviceName, final String operationName, final String endpointName,
            final String mep, final String content, final List<File> attachmentFiles,
            final Map<String, String> properties, final Subject subject)
            throws PerformActionErrorException, ChannelClientDoesNotExistException {
        try {

            NotificationFilterSupport myFilter = new NotificationFilterSupport();

            this.mBeanServerConnection.addNotificationListener(new ObjectName(
                    "Petals:name="+SAMPLECLIENT_JMX_NAME+",type=service"), callBackListener, null, null);

            Object[] objectsLoad = new Object[] { interfaceName, serviceName, operationName,
                    endpointName, mep, content, attachmentFiles, properties, subject };
            String[] stringsLoad = new String[] { "java.lang.String", "java.lang.String",
                    "java.lang.String", "java.lang.String", "java.lang.String", "java.lang.String",
                    "java.util.List", "java.util.Map", "javax.security.auth.Subject" };

            String petalsMeId = (String) this.performAction(SEND_ASYNC, objectsLoad, stringsLoad);
            return petalsMeId;

        } catch (InstanceNotFoundException e) {
            throw new ChannelClientDoesNotExistException("Can't find sampleclient bean", e);
        } catch (MalformedObjectNameException e) {
            throw new ChannelClientDoesNotExistException(e);
        } catch (IOException e) {
            throw new ChannelClientDoesNotExistException(e);
        }

    }

    /**
     * Remove a listener from JMX server ; strongly recommended after an asynch
     * call.
     * 
     * @param listener
     * @throws ChannelClientDoesNotExistException
     */
    public void removeNotificationListener(NotificationListener listener)
            throws ChannelClientDoesNotExistException {
        try {

            this.mBeanServerConnection.removeNotificationListener(new ObjectName(
                    "Petals:name=SampleClient,type=service"), listener);

        } catch (ListenerNotFoundException e) {
            throw new ChannelClientDoesNotExistException(e);
        } catch (NullPointerException e) {
            throw new ChannelClientDoesNotExistException(e);
        } catch (InstanceNotFoundException e) {
            throw new ChannelClientDoesNotExistException(e);
        } catch (MalformedObjectNameException e) {
            throw new ChannelClientDoesNotExistException(e);
        } catch (IOException e) {
            throw new ChannelClientDoesNotExistException(e);
        }
    }

    /**
     * Send a message to PEtALS synchronously
     * 
     * @param interfaceName
     * @param serviceName
     * @param operationName
     * @param endpointName
     * @param mep
     * @param content
     * @param attachmentFiles
     * @param properties
     * @param subject
     * @throws PerformActionErrorException
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> sendSync(final String interfaceName, final String serviceName,
            final String operationName, final String endpointName, final String mep,
            final String content, final List<File> attachmentFiles,
            final Map<String, String> properties, final Subject subject)
            throws PerformActionErrorException {

        Object[] objectsLoad = new Object[] { interfaceName, serviceName, operationName,
                endpointName, mep, content, attachmentFiles, properties, subject };
        String[] stringsLoad = new String[] { "java.lang.String", "java.lang.String",
                "java.lang.String", "java.lang.String", "java.lang.String", "java.lang.String",
                "java.util.List", "java.util.Map", "javax.security.auth.Subject" };

        Object result = this.performAction(SEND_SYNC, objectsLoad, stringsLoad);
        if (!(result instanceof Map)) {
            throw new PerformActionErrorException("Unexpected result type");
        } else {
            return (Map<String, Object>) result;
        }

    }

}
